package Algorithms;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import ProblemSetUp.Problem;
import State.State;

public abstract class Algorithm {
	int steps = 0;
	Set<State> opened;
	Problem problem;
	Set<State> used;
	String failureText;
	int number;
	int neighborSteps;

	public Algorithm(State state, int neighborSteps) {
		this.opened = new HashSet<State>();
		this.used = new HashSet<State>();
		this.problem = new Problem(state, neighborSteps);
		this.neighborSteps = neighborSteps;
	}

	// Method for getting the best possible not used neighbor state.
	public State bestNeighbor() {
		Set<State> neighbors = new HashSet<State>(problem.getNeighbors());
		neighbors.removeAll(used);
		List<State> neighborsList = new ArrayList<State>(neighbors);
		Collections.sort(neighborsList);
		if (neighborsList.isEmpty())
			return null;
		return neighborsList.get(0);
	}

	// Choosing a random state from opened states that have not been used yet.
	public State chooseRandomOpened() {
		Set<State> openedNotUsed = new HashSet<State>(opened);
		openedNotUsed.removeAll(used);
		if (openedNotUsed.size() == 0)
			return null;
		Random rand1 = new Random();
		int rand = rand1.nextInt(openedNotUsed.size());
		return (new ArrayList<State>(openedNotUsed)).get(rand);
	}

	// Generate random new state.
	public State randomNew() {
		return this.problem.getCurrent().generateNew();
	}

	// Choosing a random state from neighbors, prioritizing neighbors that have
	// not been used yet.
	public State randomeNeighbor() {
		Set<State> neighborsNotUsed = new HashSet<State>(problem.getNeighbors());
		neighborsNotUsed.removeAll(used);
		Random rand1 = new Random();
		if (neighborsNotUsed.size() == 0) {
			int rand = rand1.nextInt(problem.getNeighbors().size());
			return new ArrayList<State>(problem.getNeighbors()).get(rand);
		}
		int rand = rand1.nextInt(neighborsNotUsed.size());
		return (new ArrayList<State>(neighborsNotUsed)).get(rand);
	}

	// Returns the state that the algorithm would take as the next state.
	public abstract String getNext();

	public int getSteps() {
		return steps;
	}

	public void setSteps(int steps) {
		this.steps = steps;
	}

	public Set<State> getOpened() {
		return opened;
	}

	public void setOpened(Set<State> opened) {
		this.opened = opened;
	}

	public Problem getCurrent() {
		return problem;
	}

	public void setCurrent(Problem current) {
		this.problem = current;
	}

	public Set<State> getUsed() {
		return used;
	}

	public void setUsed(Set<State> used) {
		this.used = used;
	}

	public int getNumber() {
		return number;
	}

	public void setNumber(int number) {
		this.number = number;
	}

	public void setFailureText(String failureText) {
		this.failureText = failureText;
	}

	public String getFailureText() {
		return failureText;
	}

	public int getNeighborSteps() {
		return neighborSteps;
	}

	public abstract double getTemp();

}
