package Algorithms;

import ProblemSetUp.Problem;
import State.PuzzleState;
import State.State;

public class SimulatedAnnealing extends Algorithm {
	double temperature = 60;
	double coolingConstant = 0.05;

	public SimulatedAnnealing(State state, int neighbor) {
		super(state, neighbor);
		this.setFailureText("Temperatuur on liiga madal, et jtkata. Lahendust ei leitud!");
		this.setNumber(4);
		if (state.getClass() == (new PuzzleState()).getClass()) {
			this.temperature = 200;
			// this.coolingConstant = 0.05;
		}
	}

	// Takes the step towards next state according to simulated annealing.
	@Override
	public String getNext() {
		State<?> neighbour = this.bestNeighbor();
		if (neighbour != null
				&& this.problem.getEvaluation() > neighbour.findEvaluation()) {
			this.setCurrent(new Problem(neighbour, this.neighborSteps));
			this.getOpened().addAll(this.getCurrent().getNeighbors());
			this.getUsed().add(neighbour);
			temperature *= 1 - coolingConstant;
			this.setSteps(this.getSteps() + 1);
			return "Samm tehti parema seisu suunas!\n\n\n\n";
		} else {
			State<?> sa = annealing();
			if (sa == null)
				return null;
			this.setCurrent(new Problem(sa, this.neighborSteps));
			this.getOpened().addAll(this.getCurrent().getNeighbors());
			this.getUsed().add(sa);
			this.setSteps(this.getSteps() + 1);
			return "Jrglaste hulgas paremat seisu ei olnud!" + '\n'
					+ "Samm allamge!\n\n\n";
		}

	}

	// Find the next accepted state.
	public State annealing() {
		int a = this.getCurrent().getEvaluation();
		while (temperature > 1) {
			State x = this.randomeNeighbor();
			int b = x.findEvaluation();
			double pr = 1 - ((b - a + 1) / temperature);
			temperature *= 1 - coolingConstant;
			if (pr > Math.random()) {
				return x;
			}
		}
		return null;
	}

	@Override
	public double getTemp() {
		return temperature;
	}
}
