package GUI;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import Algorithms.BackTracking;
import Algorithms.RandomRestart;
import Algorithms.RegularHillClimb;
import Algorithms.SimulatedAnnealing;
import ProblemSetUp.Statistics;
import State.PuzzleState;
import State.QueensState;
import State.State;

public class AlgorithmChoice extends JPanel {
	JButton backButton;
	int neighborSteps;
	int problem;
	State state;
	Statistics statistics;
	String init = "Genereeritud algseis!\n\n\n\n ";

	// Nupud
	JButton regularHillClimb = new JButton("Tavaline mkketus");
	JButton backTracking = new JButton(
			"<html>Mkketus<br />tagasiprdumisega</html>");
	JButton randomRestart = new JButton(
			"<html>Mkketus juhusliku <br />uuestialustamisega</html>");
	JButton simulatedAnnealing = new JButton("Libalmutamine");
	JButton statisticsDisplay = new JButton("Statistika");

	public AlgorithmChoice(JButton tag, final int problem, State givenState,
			int neighborSt) {
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		// Initializing fields.
		this.neighborSteps = neighborSt;
		this.statistics = new Statistics(problem * 10 + neighborSteps);
		if (givenState != null) {
			this.state = givenState;
			init = "Eelnevalt kasutatud algseis!\n\n\n\n";
		} else
			this.state = problem == 1 ? new QueensState() : new PuzzleState();
		this.problem = problem;
		this.backButton = tag;

		// Setting up components and adding action listeners.
		final String[] options;
		if (problem == 1)
			options = new String[] {
					"Jrglasteks 1 kigu kaugusel olevad seisud",
					"Jrglasteks 2 kigu kaugusel olevad seisud" };
		else
			options = new String[] {
					"Jrglasteks 1 kigu kaugusel olevad seisud",
					"Jrglasteks 2 kigu kaugusel olevad seisud",
					"Jrglasteks 6 kigu kaugusel olevad seisud" };
		JComboBox<String> neighborChoice = new JComboBox<>(options);
		neighborChoice
				.setSelectedIndex(neighborSteps != 6 ? (neighborSteps - 1) : 2);
		neighborChoice.setMaximumSize(new Dimension(300, 30));
		neighborChoice.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				@SuppressWarnings("unchecked")
				JComboBox<String> cb = (JComboBox<String>) e.getSource();
				String kirje = (String) cb.getSelectedItem();
				neighborSteps = (Integer.parseInt(kirje.split(" ")[1]));
				statistics = new Statistics(problem * 10 + neighborSteps);
			}
		});
		regularHillClimb.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeAll();
				add(new ProcessDisplay(state, new RegularHillClimb(state,
						neighborSteps), backButton, problem, statistics, init));
				revalidate();
				repaint();
			}
		});

		backTracking.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeAll();
				add(new ProcessDisplay(state, new BackTracking(state,
						neighborSteps), backButton, problem, statistics, init));
				revalidate();
				repaint();
			}
		});

		randomRestart.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeAll();
				add(new ProcessDisplay(state, new RandomRestart(state,
						neighborSteps), backButton, problem, statistics, init));
				revalidate();
				repaint();
			}
		});

		simulatedAnnealing.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeAll();
				add(new ProcessDisplay(state, new SimulatedAnnealing(state,
						neighborSteps), backButton, problem, statistics, init));
				revalidate();
				repaint();
			}
		});

		statisticsDisplay.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final JComboBox<String> statistikaVariandid = new JComboBox<>(
						options);
				statistikaVariandid
						.setSelectedIndex(neighborSteps != 6 ? (neighborSteps - 1)
								: 2);
				statistikaVariandid.setMaximumSize(new Dimension(300, 30));
				statistikaVariandid.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						removeAll();
						add(statistikaVariandid);
						@SuppressWarnings("unchecked")
						JComboBox<String> cb = (JComboBox<String>) e
								.getSource();
						String kirje = (String) cb.getSelectedItem();
						int valik = (Integer.parseInt(kirje.split(" ")[1]));
						displayStatistics(valik);

					}
				});
				removeAll();
				add(statistikaVariandid);
				displayStatistics(neighborSteps);
			}
		});

		regularHillClimb.setAlignmentX(Component.CENTER_ALIGNMENT);
		regularHillClimb.setMaximumSize(new Dimension(200, 60));
		backTracking.setAlignmentX(Component.CENTER_ALIGNMENT);
		backTracking.setMaximumSize(new Dimension(200, 60));
		randomRestart.setAlignmentX(Component.CENTER_ALIGNMENT);
		randomRestart.setMaximumSize(new Dimension(200, 60));
		simulatedAnnealing.setAlignmentX(Component.CENTER_ALIGNMENT);
		simulatedAnnealing.setMaximumSize(new Dimension(200, 60));
		statisticsDisplay.setAlignmentX(Component.CENTER_ALIGNMENT);
		statisticsDisplay.setMaximumSize(new Dimension(200, 60));
		JTextArea infoArea = new JTextArea();
		infoArea.setMaximumSize(new Dimension(400, 150));
		infoArea.setWrapStyleWord(true);
		infoArea.setLineWrap(true);
		infoArea.setEditable(false);
		if (problem == 1)
			infoArea.setText("8 lipu probleem - malelauale tuleb paigutada 8 lippu nii, et kski neist teisele tuld ei annaks. "
					+ "See thendab, et igas reas, igas veerus, igas diagonaalis on maksimaalselt ks lipp.\n Antud "
					+ "programmi puhul on genereeritud algseisuks seis, kus igasse veergu on juhuslikult asetatud "
					+ "ks lipp. Seisu he kigu kaugusel olevateks jrglasteks on kik sellised seisud,"
					+ " mis saadavad sellest seisust "
					+ "he lipu veeru piires liigutamisega.\n \n"
					+ "Vali algoritm!");
		else {
			infoArea.setText("8-mng - eesmrgiks on numbrid igesse kohta liigutada, see thendab esimeses ruudus number 1, teises "
					+ "number 2 ja nii edasi. Thi ruut peaks lahenduses asetsema viimasel kohal."
					+ "Iga kiguga saab thja ruudu vahetada selle krval (all, leval, paremal, vasakul) oleva numbriruuduga.\n Antud "
					+ "programmi puhul on genereeritud algseisuks seis, kus numbrid on kohtadele juhuslikult asetatud. "
					+ "Seisu he kigu kaugusel olevateks jrglasteks on kik sellised seisud, mis saadavad sellest seisust "
					+ "he lubatud kiguga.\n \n" + "Vali algoritm!");
			infoArea.setMaximumSize(new Dimension(400, 190));
		}

		// Adding components.
		this.add(infoArea);
		this.add(neighborChoice);
		this.add(regularHillClimb);
		this.add(backTracking);
		this.add(randomRestart);
		this.add(simulatedAnnealing);
		this.add(statisticsDisplay);
		add(backButton);
	}

	// Method for displaying the statistics of algorithms performance.
	public void displayStatistics(int valik) {
		JTextArea statistics = new JTextArea();
		statistics.setMaximumSize(new Dimension(350, 500));
		statistics.setEditable(false);
		try (BufferedReader br = new BufferedReader(new InputStreamReader(
				new FileInputStream(findStatitsicsFile(valik))))) {
			statistics.read(br, null);
		} catch (IOException e1) {
		}
		add(statistics);
		final JButton menu = new JButton("Tagasi mensse");
		menu.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeAll();
				add(new AlgorithmChoice(backButton, problem, state,
						neighborSteps));
				revalidate();
				repaint();
			}
		});
		add(menu);
		revalidate();
		repaint();
	}

	public File findStatitsicsFile(int valik) {
		String failiNimi = "Statistics" + problem + valik + ".txt";
		return new File(failiNimi);
	}
}
