package GUI;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import Algorithms.Algorithm;
import Algorithms.BackTracking;
import Algorithms.RandomRestart;
import Algorithms.RegularHillClimb;
import Algorithms.SimulatedAnnealing;
import ProblemSetUp.Statistics;
import State.PuzzleState;
import State.QueensState;
import State.State;

public class CollectingStatistics extends JFrame {
	static int nrOfRepeats;

	public static void main(String[] args) {
		final CollectingStatistics mainFrame = new CollectingStatistics();
		final Container container = mainFrame.getContentPane();
		container.setLayout(new FlowLayout());
		mainFrame.setSize(320, 400);
		mainFrame.setLocation(100, 100);
		mainFrame.setTitle("Kogu statistikat");

		final JTextArea area = new JTextArea(15, 25);
		area.setText("See programm kogub mkketusu programmi jaoks statistikat. Sisesta kasti number, mitme seisu "
				+ "pealt statistikat koguda soovid, ning vajuta OK. Seejrel genereerib programm vastavalt nii palju algseise,"
				+ " ning kasutab iga algseisu peal igat algoritmi kigi jrglaste valiku variantidega. Tulemused salvestatakse statistika"
				+ "failidesse.\n\n NB! Kui kaustas, kus programm asub, juba on genereeritud statistikafailid, siis lisatakse statistika nendesse failidesse juurde. "
				+ "Kui soovid statistika salvestada eraldi failidesse, tsta programmi t ajaks olemasolevad statistikafailid erinevasse kausta!"
				+ "\n\nStatistika kogumine vtab aega! Kasuta aega targasti ja tegele kogumise ajal muude asjadega. Programm annab teada, kui on t lpetanud.");
		area.setEditable(false);
		area.setWrapStyleWord(true);
		area.setLineWrap(true);

		final JTextField nrOfStates = new JTextField() {
			@Override
			public void processKeyEvent(KeyEvent ev) {
				char c = ev.getKeyChar();
				try {
					if (c > 31 && c < 127) {
						Integer.parseInt(c + "");
					}
					super.processKeyEvent(ev);
				}

				catch (NumberFormatException nfe) {

				}
			}
		};
		nrOfStates.setColumns(6);

		final JButton okButton = new JButton("OK");
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!nrOfStates.getText().equals("")
						&& Integer.parseInt(nrOfStates.getText()) > 0) {
					nrOfRepeats = Integer.parseInt(nrOfStates.getText());
					int a = 0;
					while (a < nrOfRepeats) {
						PuzzleState puzzle = new PuzzleState();
						makeStateStatistics(1, puzzle, 2);
						makeStateStatistics(2, puzzle, 2);
						makeStateStatistics(6, puzzle, 2);
						a++;
					}
					a = 0;
					while (a < nrOfRepeats) {
						QueensState queen = new QueensState();
						makeStateStatistics(1, queen, 1);
						makeStateStatistics(2, queen, 1);
						a++;
					}
					JOptionPane
							.showMessageDialog(
									null,
									"T tehtud!\nKogutud statistikaga tutvumiseks vaata statistika faile, mis on programmiga samas kaustas,\n"
											+ "vi ava samas kaustas mkketusu programm ja vaata statistikat selle abiga.\n",
									"Valmis!", 1);
				} else
					nrOfStates.setText("");
			}
		});

		final JButton exitButton = new JButton("Vlju");
		exitButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		exitButton.setMaximumSize(new Dimension(200, 80));
		exitButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);

			}
		});

		container.add(area);
		container.add(Box.createRigidArea(new Dimension(50, 10)));
		container.add(nrOfStates);
		container.add(okButton);
		container.add(Box.createRigidArea(new Dimension(50, 10)));
		container.add(exitButton);

		mainFrame.setVisible(true);
		mainFrame.setDefaultCloseOperation(EXIT_ON_CLOSE);
		mainFrame.setResizable(true);
		mainFrame.setLocationRelativeTo(null);

	}

	public static void makeStateStatistics(int neighborSteps, State state,
			int prNr) {
		Statistics statistics = new Statistics(10 * prNr + neighborSteps);
		RegularHillClimb regular = new RegularHillClimb(state, neighborSteps);
		BackTracking backTrack = new BackTracking(state, neighborSteps);
		RandomRestart restart = new RandomRestart(state, neighborSteps);
		SimulatedAnnealing annealing = new SimulatedAnnealing(state,
				neighborSteps);
		makeAlgorithmStat(regular, statistics);
		makeAlgorithmStat(backTrack, statistics);
		makeAlgorithmStat(restart, statistics);
		makeAlgorithmStat(annealing, statistics);
	}

	public static void makeAlgorithmStat(Algorithm alg, Statistics stat) {
		while (alg.getCurrent().getEvaluation() != 0 && alg.getNext() != null)
			;
		if (alg.getCurrent().getEvaluation() == 0)
			stat.updateStatistics(alg.getNumber(), alg.getSteps(), alg
					.getOpened().size());
		else
			stat.updateStatistics(alg.getNumber(), alg.getSteps(), alg
					.getOpened().size(), false);
	}

}
