package GUI;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import Algorithms.Algorithm;
import Algorithms.SimulatedAnnealing;
import ProblemSetUp.Problem;
import ProblemSetUp.Statistics;
import State.State;

public class ProcessDisplay extends JPanel {
	State initial;
	Problem problem;
	Algorithm algorithm;
	JButton backButton;
	int prNr;
	JTextArea infoBox = new JTextArea();
	boolean finished = false;
	Statistics statistics;
	final JButton nextButton = new JButton("Edasi");
	JButton finish = new JButton("Tee lpuni");

	public ProcessDisplay(State state, Algorithm alg, JButton tag, int prNr,
			Statistics stat, String init) {
		// Setting up buttons and initializing fields.
		this.statistics = stat;
		this.initial = state;
		this.prNr = prNr;
		this.backButton = tag;
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.problem = alg.getCurrent();
		this.algorithm = alg;

		infoBox.setText(init);
		infoBox.setWrapStyleWord(true);
		infoBox.setLineWrap(true);
		infoBox.setEditable(false);
		infoBox.setEditable(false);
		infoBox.setMaximumSize(new Dimension(300, 100));

		nextButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!finished) {
					stepOfAlgorithm();
				}
			}
		});

		finish.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int i = 1;
				while (!finished) {
					stepOfAlgorithm();
				}
			}
		});
		updateDisplay();
	}

	// Method for making the algorithm take a step towards next state.
	// Information on how the step was taken is given in info box.
	public void stepOfAlgorithm() {
		String infoText = algorithm.getNext();
		if (infoText == null) {
			finished = true;
			statistics.updateStatistics(algorithm.getNumber(),
					algorithm.getSteps(), algorithm.getOpened().size(), false);
			finish(false);
			return;
		}
		infoBox.setText(infoText);
		problem = algorithm.getCurrent();
		updateDisplay();
		if (problem.getEvaluation() == 0) {
			statistics.updateStatistics(algorithm.getNumber(),
					algorithm.getSteps(), algorithm.getOpened().size());
			finish(true);
			finished = true;
		}
	}

	// updates the display with board for new state and other indicators.
	public void updateDisplay() {
		JPanel board = problem.getCurrent().display();
		JLabel hinnang = new JLabel();
		hinnang.setText("Hinnang on " + problem.getEvaluation());
		removeAll();
		add(board);
		add(infoBox);
		add(hinnang);
		if (algorithm.getClass() == (new SimulatedAnnealing(initial, 1))
				.getClass()) {
			add(new JLabel("Temperatuur on "
					+ (Math.round(algorithm.getTemp() * 1000) / 1000D)));

		}
		add(buttonPanel());
		revalidate();
		repaint();
	}

	// Returns a panel with buttons.
	public JPanel buttonPanel() {
		final JButton menuu = new JButton("Tagasi mensse");
		menuu.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeAll();
				add(new AlgorithmChoice(backButton, prNr, null, algorithm
						.getNeighborSteps()));
				revalidate();
				repaint();
			}
		});
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout());
		panel.add(menuu);
		panel.add(nextButton);
		panel.add(finish);
		panel.setMaximumSize(new Dimension(400, 80));
		return panel;
	}

	// Generates option pane with suitable text and option for the user to reuse
	// the state.
	public int reUsingStateOption(boolean success) {
		Object[] options = { "Jah!", "Ei!" };
		String text = "";
		String pealkiri = "Lahendus leitud";
		if (!success) {
			text = algorithm.getFailureText();
		} else {
			infoBox.setText((infoBox.getText().split("\n"))[0] + '\n'
					+ "Lahendus leitud!\n\n\n");
			text = "Lahendus leitud!";
		}
		String statSteps = "Samme tehti: " + algorithm.getSteps();
		String statOpened = "Tippe avati: " + algorithm.getOpened().size();
		String message = text + '\n' + statSteps + '\n' + statOpened + '\n';
		int n = JOptionPane
				.showOptionDialog(
						new JFrame(),
						message
								+ "\nKas soovid sama algseisu kasutada mne teise algoritmiga?",
						pealkiri, JOptionPane.YES_NO_OPTION,
						JOptionPane.QUESTION_MESSAGE, null, options, options[0]);
		return n;
	}

	// When algorithm has finished the process then displays the above option
	// pane and
	// if the user wants to reuse, acts accordingly.
	public void finish(boolean success) {
		int n = reUsingStateOption(success);
		if (n == 0) {
			removeAll();
			add(new AlgorithmChoice(backButton, prNr, initial,
					algorithm.getNeighborSteps()));
			revalidate();
			repaint();
		}
	}

}
