package ProblemSetUp;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

public class Statistics {
	File file;
	String fileName;

	// Creating or accessing the file where the statistics would be written.
	public Statistics(int i) {
		fileName = "Statistics" + i + ".txt";
		file = new File((fileName));
		if (!file.isFile()) {
			try {
				file.createNewFile();
				build();
			} catch (IOException e) {
			}
		}
	}

	// Method to write the given line into the given file.
	public void writeToFile(File fail, String a) {
		try (PrintWriter pw = new PrintWriter(new OutputStreamWriter(
				new FileOutputStream(fail, true)))) {
			pw.println(a);
		} catch (FileNotFoundException e) {
		}
	}

	// Increases the value in given String by one.
	public Object[] increaseLine(String line, int addition) {
		String[] parts = line.split(": ");
		int i = Integer.parseInt(parts[1]);
		return (new Object[] { i, parts[0] + ": " + (i + addition) });
	}

	// Computes the average given the line with previous average, the number of
	// items it was computed from
	// and the added value.
	public String computeAverage(String line, int numberOfSolved, int value) {
		String[] parts = line.split(": ");
		NumberFormat format = NumberFormat.getInstance(Locale.FRANCE);
		Number number = 0;
		try {
			number = format.parse(parts[1]);
		} catch (ParseException e) {
			e.printStackTrace();
		}
		double d = number.doubleValue();
		Double i = (d * numberOfSolved + value) / (double) (numberOfSolved + 1);
		i = Math.round(i * 100) / 100.0;
		return parts[0] + ": " + format.format(i);
	}

	// Update statistics when successfully found result.
	public void updateStatistics(int algorithmNumber, int steps, int nrOfOpened) {
		File temp = new File("temp.txt");
		try (BufferedReader reader = new BufferedReader(new InputStreamReader(
				new FileInputStream(file)))) {
			int i = 0;
			try {
				while (i < 8 * (algorithmNumber - 1) + 1) {
					writeToFile(temp, reader.readLine());
					i++;
				}
				String algorithmUsedLine = reader.readLine();
				writeToFile(temp,
						(String) increaseLine(algorithmUsedLine, 1)[1]);
				String solvedLine = reader.readLine();
				int nrOfSolved = (int) increaseLine(solvedLine, 1)[0];
				writeToFile(temp, (String) increaseLine(solvedLine, 1)[1]);
				String averageStepsLine = reader.readLine();
				writeToFile(temp,
						computeAverage(averageStepsLine, nrOfSolved, steps));
				String averageOpenedLine = reader.readLine();
				writeToFile(
						temp,
						computeAverage(averageOpenedLine, nrOfSolved,
								nrOfOpened));
				String line = "";
				while ((line = reader.readLine()) != null) {
					writeToFile(temp, line);
				}

			} catch (IOException e) {
			}
		} catch (IOException e1) {
		}
		file.delete();
		file = new File(fileName);
		temp.renameTo(file);
	}

	// Update statistics when failed to find result.
	public void updateStatistics(int algorithmNumber, int steps,
			int nrOfOpened, boolean failure) {
		File temp = new File("temp.txt");
		try (BufferedReader reader = new BufferedReader(new InputStreamReader(
				new FileInputStream(file)))) {
			int i = 0;
			try {
				while (i < 8 * (algorithmNumber - 1) + 1) {
					writeToFile(temp, reader.readLine());
					i++;
				}
				String algorithmUsedLine = reader.readLine();
				writeToFile(temp,
						(String) increaseLine(algorithmUsedLine, 1)[1]);
				String solvedLine = reader.readLine();
				writeToFile(temp, solvedLine);
				int nrOfNotSolved = (Integer.parseInt(algorithmUsedLine
						.split(": ")[1]) + 1)
						- (Integer.parseInt(solvedLine.split(": ")[1]));
				writeToFile(temp, reader.readLine());
				writeToFile(temp, reader.readLine());
				String averageStepsLine = reader.readLine();
				writeToFile(temp,
						computeAverage(averageStepsLine, nrOfNotSolved, steps));
				String averageOpenedLine = reader.readLine();
				writeToFile(
						temp,
						computeAverage(averageOpenedLine, nrOfNotSolved,
								nrOfOpened));
				String line = "";
				while ((line = reader.readLine()) != null) {
					writeToFile(temp, line);
				}
			} catch (IOException e) {
			}
		} catch (IOException e1) {
		}
		file.delete();
		file = new File(fileName);
		temp.renameTo(file);
	}

	// Methods for building the basic structure for statistics file.
	public void build() {
		buildBlock("Tavaline mkketus"); // 1
		buildBlock("Mkketus tagasiliikumisega"); // 2
		buildBlock("Mkketus juhusliku uuestialustamisega"); // 3
		buildBlock("Libalmutamine"); // 4
	}

	public void buildBlock(String algoritm) {
		writeToFile(file, algoritm);
		writeToFile(file, "Algoritmi kasutatud: 0");
		writeToFile(file, "Lahendus leitud: 0");
		writeToFile(file, "Lahenduse leidmisel keskmiselt samme tehtud: 0");
		writeToFile(file, "Lahenduse leidmisel keskmiselt tippe avatud: 0");
		writeToFile(file, "Lahenduse mitteleidmisel keskmiselt samme tehtud: 0");
		writeToFile(file, "Lahenduse mitteleidmisel keskmiselt tippe avatud: 0");
		writeToFile(file, "");

	}

}
