package State;

public class PuzzleEvaluation {

	// Evaluation function for 8-puzzle state
	public static int evaluate(PuzzleState puzzleState) {
		int steps = 0;
		GameState gs = new GameState(puzzleState);
		for (int i = 1; i < 9; i++) {
			steps += gs.countSteps(i);
		}
		return steps;
	}

}

// Side class, puzzleState will be transformed to GameState that consists of
// number Tiles. This will ease
// calculating the evaluation for 8-puzzle state.
class GameState {
	Tile[] state;
	GameState previous = null;

	public GameState(Tile[] state) {
		this.state = state;
	}

	public GameState(PuzzleState pz) {
		int[][] order = pz.getCurrentState();
		Tile[] state = new Tile[9];
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				Tile tile = new Tile(new int[] { i, j }, order[i][j]);
				state[i * 3 + j] = tile;
			}
		}
		this.state = state;
	}

	// Method for swapping two tiles in the state (on the board).
	public void swapTiles(Tile tile1, Tile tile2) {
		if (tile1.isEmpty() || tile2.isEmpty()) {
			int[] newLocation = tile1.location;
			tile1.location = tile2.location;
			tile2.location = newLocation;
		}
	}

	// Method that given a tile number will count the steps it would take to
	// move
	// that tile to it's goal position in a simple manner (taking into
	// consideration the rule
	// that only empty tile can be moved). This method doesn't give the lower
	// bound though, as
	// it first moves tile to the right row, then to right column. (Might be
	// faster to alternate between
	// swapping towards right row, right column every turn.
	public int countSteps(int tileNr) {
		Tile tile = tileWithNr(tileNr);
		int steps = 0;
		for (int[] direction = tile.getDirection(); direction[0] != 0
				|| direction[1] != 0; direction = tile.getDirection()) {
			if (direction[0] != 0) {
				if (empty().location[0] != tile.location[0] + direction[0]
						|| empty().location[1] != tile.location[1]) {
					if (empty().location[1] == tile.location[1]) {
						steps++;
						if (empty().location[1] > 0) {
							swapTiles(empty(), tileOnLocation(new int[] {
									empty().location[0],
									empty().location[1] - 1 }));
						} else
							swapTiles(empty(), tileOnLocation(new int[] {
									empty().location[0],
									empty().location[1] + 1 }));
					}
					steps += moveEmptyToRow(tile.location[0] + direction[0]);
					steps += moveEmptyToColumn(tile.location[1]);
				}
				// swap our tile with the empty space.
				steps++;
				swapTiles(empty(), tile);
			} else { // direction[1] must be != 0
				if (empty().location[0] != tile.location[0]
						|| empty().location[1] != tile.location[1]
								+ direction[1]) {
					if (empty().location[0] == tile.location[0]) {
						steps++;
						if (empty().location[0] > 0) {
							swapTiles(empty(), tileOnLocation(new int[] {
									empty().location[0] - 1,
									empty().location[1] }));
						} else
							swapTiles(empty(), tileOnLocation(new int[] {
									empty().location[0] + 1,
									empty().location[1] }));
					}
					steps += moveEmptyToColumn(tile.location[1] + direction[1]);
					steps += moveEmptyToRow(tile.location[0]);
				}
				// swap our tile with the empty space.
				steps++;
				swapTiles(empty(), tile);
			}
		}
		return steps;
	}

	// Moves empty tile to given row while counting steps it took to get there.
	public int moveEmptyToRow(int row) {
		int steps = 0;
		for (int[] emptyLocation = empty().location; emptyLocation[0] != row; emptyLocation = empty().location) {
			steps++;
			if (emptyLocation[0] > row) {
				swapTiles(empty(), tileOnLocation(new int[] {
						emptyLocation[0] - 1, emptyLocation[1] }));
			} else
				swapTiles(empty(), tileOnLocation(new int[] {
						emptyLocation[0] + 1, emptyLocation[1] }));
		}
		return steps;
	}

	// Moves empty tile to given column while counting steps it took to get
	// there.
	public int moveEmptyToColumn(int column) {
		int steps = 0;
		for (int[] emptyLocation = empty().location; emptyLocation[1] != column; emptyLocation = empty().location) {
			steps++;
			if (emptyLocation[1] > column) {
				swapTiles(empty(), tileOnLocation(new int[] { emptyLocation[0],
						emptyLocation[1] - 1 }));
			} else
				swapTiles(empty(), tileOnLocation(new int[] { emptyLocation[0],
						emptyLocation[1] + 1 }));
		}
		return steps;
	}

	// Find the empty tile in the state (array of tiles).
	public Tile empty() {
		for (Tile tile : state) {
			if (tile.isEmpty())
				return tile;
		}
		return new Tile();
	}

	// Returns the tile on given coordinates.
	public Tile tileOnLocation(int[] location) {
		for (Tile tile : state) {
			if (tile.location[0] == location[0]
					&& tile.location[1] == location[1])
				return tile;
		}
		return new Tile();
	}

	// Returns the tile based on its number.
	public Tile tileWithNr(int tileNr) {
		for (Tile tile : state) {
			if (tile.number == tileNr)
				return tile;
		}
		return new Tile();
	}

}
