package State;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import javax.swing.JPanel;

import GUI.BoardDisplay;

public class PuzzleState extends State<PuzzleState> {
	int[] emptyLocation;

	public PuzzleState(int[][] state) {
		super(state);
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				if (state[i][j] == 0)
					this.emptyLocation = new int[] { i, j };
			}
		}
	}

	// Constructor generating random new state
	public PuzzleState() {
		int[][] state = new int[3][3];
		boolean isSolvable = false;
		while (!isSolvable) {
			List<Integer> numbers = new ArrayList<Integer>();
			for (int i = 0; i < 9; i++) {
				numbers.add(i);
			}
			state = new int[3][3];
			// Keep the order of numbers in the state as a one dimensional array
			// as well, to
			// ease the calculation of inversions
			int[] order = new int[9];
			for (int i = 0; i < 3; i++) {
				for (int j = 0; j < 3; j++) {
					Random rand = new Random();
					int number = rand.nextInt(numbers.size());
					if (numbers.get(number) == 0)
						emptyLocation = new int[] { i, j };
					state[i][j] = numbers.get(number);
					order[3 * i + j] = numbers.remove(number);
				}
			}
			if (findInversions(order) % 2 == 0) {
				isSolvable = true;
			}
		}
		this.setCurrentState(state);
		this.setPrevious(null);

	}

	// copy constructor
	public PuzzleState(PuzzleState oldState) {
		int[][] currentState = oldState.getCurrentState();
		int[][] state = new int[currentState.length][currentState.length];
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				state[i][j] = currentState[i][j];
			}
		}
		this.emptyLocation = oldState.emptyLocation;
		this.setCurrentState(state);
		this.setPrevious(oldState);
	}

	// copy constructor 2
	public PuzzleState(PuzzleState oldState, PuzzleState parent) {
		int[][] currentState = oldState.getCurrentState();
		int[][] state = new int[currentState.length][currentState.length];
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				state[i][j] = currentState[i][j];
			}
		}
		this.emptyLocation = oldState.emptyLocation;
		this.setCurrentState(state);
		this.setPrevious(parent);
	}

	// Counts inversions given an array of numbers
	public int findInversions(int[] order) {
		int inversions = 0;
		for (int i = 0; i < order.length; i++) {
			int number = order[i];
			int j = i;
			for (; j < order.length; j++) {
				if (number > order[j] && number != 0 && order[j] != 0) {
					inversions++;
				}
			}
		}
		return inversions;
	}

	// Following methods are to swap empty tile with one up/down/right/left from
	// it.
	public void moveEmptyUp() {
		try {
			int[][] currentState = this.getCurrentState();
			currentState[emptyLocation[0]][emptyLocation[1]] = currentState[emptyLocation[0] - 1][emptyLocation[1]];
			currentState[emptyLocation[0] - 1][emptyLocation[1]] = 0;
			this.setCurrentState(currentState);
			this.emptyLocation = new int[] { emptyLocation[0] - 1,
					emptyLocation[1] };
		} catch (ArrayIndexOutOfBoundsException e) {
		}
	}

	public void moveEmptyDown() {
		try {
			int[][] currentState = this.getCurrentState();
			currentState[emptyLocation[0]][emptyLocation[1]] = currentState[emptyLocation[0] + 1][emptyLocation[1]];
			currentState[emptyLocation[0] + 1][emptyLocation[1]] = 0;
			this.setCurrentState(currentState);
			this.emptyLocation = new int[] { emptyLocation[0] + 1,
					emptyLocation[1] };
		} catch (ArrayIndexOutOfBoundsException e) {
		}
	}

	public void moveEmptyRight() {
		try {
			int[][] currentState = this.getCurrentState();
			currentState[emptyLocation[0]][emptyLocation[1]] = currentState[emptyLocation[0]][emptyLocation[1] + 1];
			currentState[emptyLocation[0]][emptyLocation[1] + 1] = 0;
			this.setCurrentState(currentState);
			this.emptyLocation = new int[] { emptyLocation[0],
					emptyLocation[1] + 1 };
		} catch (ArrayIndexOutOfBoundsException e) {
		}
	}

	public void moveEmptyLeft() {
		try {
			int[][] currentState = this.getCurrentState();
			currentState[emptyLocation[0]][emptyLocation[1]] = currentState[emptyLocation[0]][emptyLocation[1] - 1];
			currentState[emptyLocation[0]][emptyLocation[1] - 1] = 0;
			this.setCurrentState(currentState);
			this.emptyLocation = new int[] { emptyLocation[0],
					emptyLocation[1] - 1 };
		} catch (ArrayIndexOutOfBoundsException e) {
		}
	}

	@Override
	public int findEvaluation() {
		return PuzzleEvaluation.evaluate(this);
	}

	// Methods for finding neighbours 1,2 and 6 steps away from given
	// PuzzleState
	public Set<PuzzleState> findNeighbors() {
		Set<PuzzleState> set = new HashSet<PuzzleState>();
		// Neighbours moving empty tile down, up, right and left
		PuzzleState down = new PuzzleState(this);
		down.moveEmptyDown();
		set.add(down);
		PuzzleState up = new PuzzleState(this);
		up.moveEmptyUp();
		set.add(up);
		PuzzleState right = new PuzzleState(this);
		right.moveEmptyRight();
		set.add(right);
		PuzzleState left = new PuzzleState(this);
		left.moveEmptyLeft();
		set.add(left);
		set.remove(this);
		return set;
	}

	@Override
	public Set<PuzzleState> findNeighbors(PuzzleState parent) {
		Set<PuzzleState> set = new HashSet<PuzzleState>();
		// Neighbours moving empty tile down, up, right and left
		PuzzleState down = new PuzzleState(this, parent);
		down.moveEmptyDown();
		set.add(down);
		PuzzleState up = new PuzzleState(this, parent);
		up.moveEmptyUp();
		set.add(up);
		PuzzleState right = new PuzzleState(this, parent);
		right.moveEmptyRight();
		set.add(right);
		PuzzleState left = new PuzzleState(this, parent);
		left.moveEmptyLeft();
		set.add(left);
		set.remove(this);
		return set;
	}

	@Override
	public Set<PuzzleState> findNeighborsTwoSteps(PuzzleState parent) {
		Set<PuzzleState> oneStepNeighbours = this.findNeighbors(this);
		Set<PuzzleState> twoStepNeighbors = new HashSet<PuzzleState>();
		for (PuzzleState neighbor : oneStepNeighbours) {
			twoStepNeighbors.addAll(neighbor.findNeighbors(parent));
			if (neighbor.findEvaluation() == 0) {
				twoStepNeighbors.add(neighbor);
			}
		}
		return twoStepNeighbors;
	}

	public Set<PuzzleState> findNeighborsFourSteps() {
		Set<PuzzleState> twoStepNeighbours = this.findNeighborsTwoSteps(this);
		Set<PuzzleState> fourStepNeighbours = new HashSet<PuzzleState>();
		for (PuzzleState neighbour : twoStepNeighbours) {
			fourStepNeighbours.addAll(neighbour.findNeighborsTwoSteps(this));
		}
		return fourStepNeighbours;
	}

	@Override
	public Set<PuzzleState> findNeighborsSixSteps() {
		Set<PuzzleState> fourStepNeighbours = this.findNeighborsFourSteps();
		Set<PuzzleState> sixStepNeighbours = new HashSet<PuzzleState>();
		for (PuzzleState neighbour : fourStepNeighbours) {
			sixStepNeighbours.addAll(neighbour.findNeighborsTwoSteps(this));
		}
		return sixStepNeighbours;
	}

	// For displaying puzzle board with current state
	@Override
	public JPanel display() {
		return new BoardDisplay(this);
	}

	@Override
	public int compareTo(PuzzleState o) {
		return this.findEvaluation() - o.findEvaluation();
	}

	@Override
	public String toString() {
		return "PuzzleState [getCurrentState()=" + '\n'
				+ Arrays.toString(getCurrentState()[0]) + '\n'
				+ Arrays.toString(getCurrentState()[1]) + '\n'
				+ Arrays.toString(getCurrentState()[2]) + "]";
	}

	@Override
	public State<PuzzleState> generateNew() {
		return new PuzzleState();
	}
}
