package State;

import java.util.HashSet;
import java.util.Random;
import java.util.Set;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import GUI.BoardDisplay;

public class QueensState extends State<QueensState> {

	public QueensState(int[][] state) {
		super(state);
	}

	// Random new generator
	public QueensState() {
		int[][] state = new int[8][8];
		for (int i = 0; i < 8; i++) {
			Random rand1 = new Random();
			int rand = rand1.nextInt(8);
			state[rand][i] = 1;
		}
		this.setCurrentState(state);
		this.setPrevious(null);
	}

	// copy constructor
	public QueensState(QueensState oldState) {
		int[][] currentState = oldState.getCurrentState();
		int[][] state = new int[8][8];
		for (int i = 0; i < 8; i++) {
			for (int j = 0; j < 8; j++) {
				state[i][j] = currentState[i][j];
			}
		}
		this.setCurrentState(state);
		this.setPrevious(oldState);
	}

	@Override
	public int findEvaluation() {
		return QueensEvaluation.evaluate(this);
	}

	// Methods for placing and taking a queen from the location on the chess
	// board.
	public void placeQueen(int a, int b) {
		this.getCurrentState()[a][b] = 1;
	}

	public void takeQueen(int a, int b) {
		this.getCurrentState()[a][b] = 0;
	}

	// Methods for finding neighbours 1 and 2 steps away from given
	// PuzzleState
	@Override
	public Set<QueensState> findNeighbors(QueensState parent) {
		Set<QueensState> list = new HashSet<QueensState>();
		for (int i = 0; i < 8; i++) {
			int marker = 0;
			while (this.getCurrentState()[marker][i] == 0)
				marker++;
			for (int j = 0; j < 8; j++) {
				if (j != marker) {
					QueensState abi = new QueensState(this);
					abi.takeQueen(marker, i);
					abi.placeQueen(j, i);
					list.add(abi);
				}
			}
		}
		return list;
	}

	@Override
	public Set<QueensState> findNeighborsTwoSteps(QueensState parent) {
		Set<QueensState> oneStepNeighbours = this.findNeighbors(parent);
		Set<QueensState> twoStepNeighbours = new HashSet<QueensState>();
		for (QueensState neighbour : oneStepNeighbours) {
			twoStepNeighbours.addAll(neighbour.findNeighbors(parent));
			if (neighbour.findEvaluation() == 0) {
				twoStepNeighbours.add(neighbour);
			}
		}
		twoStepNeighbours.remove(this);
		return twoStepNeighbours;
	}

	@Override
	public Set<QueensState> findNeighborsSixSteps() {
		JOptionPane
				.showMessageDialog(
						null,
						"Kui kik igesti ttab, siis seda hoiatust kasutaja ei ne!\n"
								+ "See hoiatus tuleneb sellest, et proovitakse 8-lipu probleemil kasutada \n"
								+ "kuue sammu kaugusel olevaid naabreid, mis aga vtab liiga palju mlu/aega \n"
								+ "ja seega programm seda ei vimalda.\n"
								+ "Palun vabandust!", "",
						JOptionPane.ERROR_MESSAGE);
		return null;
	}

	// For displaying chess board with current state
	@Override
	public JPanel display() {
		return new BoardDisplay(this);
	}

	@Override
	public int compareTo(QueensState o) {
		return this.findEvaluation() - o.findEvaluation();
	}

	@Override
	public State<QueensState> generateNew() {
		return new QueensState();
	}
}
