package State;

import java.util.Arrays;
import java.util.Set;

import javax.swing.JPanel;

public abstract class State<T extends State<T>> implements Comparable<T> {
	private int[][] currentState;
	private T previous;

	public State(int[][] state) {
		this.setCurrentState(state);
	}

	public State() {
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == this)
			return true;
		if (!(obj instanceof State))
			return false;
		State<?> state = (State<?>) obj;
		for (int i = 0; i < currentState.length; i++) {
			for (int j = 0; j < currentState[i].length; j++)
				if (this.currentState[i][j] != state.currentState[i][j]) {
					return false;
				}
		}
		return true;
	}

	public abstract State<T> generateNew();

	public abstract Set<T> findNeighbors(T parent);

	public abstract Set<T> findNeighborsTwoSteps(T parent);

	public abstract Set<T> findNeighborsSixSteps();

	public abstract JPanel display();

	public abstract int findEvaluation();

	@Override
	public int hashCode() {
		return this.findEvaluation();
	}

	public int[][] getCurrentState() {
		return currentState;
	}

	public void setCurrentState(int[][] currentState) {
		this.currentState = currentState;
	}

	public T getPrevious() {
		return previous;
	}

	public void setPrevious(T previous) {
		this.previous = previous;
	}

	@Override
	public String toString() {
		return "State [currentState=" + Arrays.toString(currentState[0])
				+ Arrays.toString(currentState[1])
				+ Arrays.toString(currentState[2]) + "]";
	}

}
