package State;

import java.util.Arrays;

// Help class for finding puzzle state evaluation. Puzzle state will be 
// represented by an array of tiles to ease the calculation.
public class Tile {
	int[] location;
	int number;

	public Tile(int[] location, int number) {
		this.location = location;
		this.number = number;
	}

	public Tile() {
		this.location = new int[] { -1, -1 };
		this.number = 9;
	}

	public void moveLeft() {
		location = new int[] { location[0], location[1] - 1 };
	}

	public void moveRight() {
		location = new int[] { location[0], location[1] + 1 };
	}

	public void moveDown() {
		location = new int[] { location[0] + 1, location[1] };
	}

	public void moveUp() {
		location = new int[] { location[0] - 1, location[1] };
	}

	public boolean isEmpty() {
		return (this.number == 0);
	}

	@Override
	public String toString() {
		return "[ " + Arrays.toString(location) + ", " + number + "]";
	}

	// Finding coords for the tile in goal state
	public int[] findGoal() {
		int[][] goalState = new int[][] { { 1, 2, 3 }, { 4, 5, 6 }, { 7, 8, 0 } };
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				if (goalState[i][j] == number)
					return new int[] { i, j };
			}
		}
		return new int[] { -1, -1 };
	}

	// Getting the direction towards which the tile should be moved to reach
	// goal.
	public int[] getDirection() {
		int[] direction = new int[2];
		int[] goal = findGoal();
		for (int i = 0; i < 2; i++) {
			if (goal[i] > location[i]) {
				direction[i] = 1;
			} else if (goal[i] < location[i]) {
				direction[i] = -1;
			}
		}
		return direction;
	}

}
